@@ -1,20 +1,28 @@
 //! Helpers to introspect Rust code when generating bindings, mostly derived by the `#[ffi_...]` macros.
 
-use crate::lang::c::{CType, Constant, ConstantValue, FnPointerType, Function, FunctionSignature, Parameter, PrimitiveType, PrimitiveValue};
+use crate::lang::c::{CType, Constant, ConstantValue, FnPointerType, Function, FunctionSignature, Parameter, PrimitiveType, PrimitiveValue, Variant};
 use std::ptr::NonNull;
 
+/// Implemented for a constant-helper produced by [`ffi_constant`](crate::ffi_constant), gives meta info for a constant.
 pub trait ConstantInfo {
     fn constant_info() -> Constant;
 }
 
+/// Implemented for a type via [`ffi_type`](crate::ffi_type), gives meta info for a type.
 pub trait CTypeInfo {
     fn type_info() -> CType;
 }
 
+/// Implemented for a function-helper produced by [`ffi_function`](crate::ffi_function), gives meta info for a function.
 pub trait FunctionInfo {
     fn function_info() -> Function;
 }
 
+/// Implemented for an enum via [`ffi_type`](crate::ffi_type) allows us to translate a variant into its meta information.
+pub trait VariantInfo {
+    fn variant_info(&self) -> Variant;
+}
+
 macro_rules! impl_ctype_primitive {
     (
         $rust_type:ty,
@@ -28,7 +36,6 @@ macro_rules! impl_ctype_primitive {
     };
 }
 
-
 macro_rules! impl_const_value_primitive {
     (
         $rust_type:ty,
@@ -152,111 +159,109 @@ where
     R: CTypeInfo,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(vec![], R::type_info());
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
 
 impl<R> CTypeInfo for Option<extern "C" fn() -> R>
-    where
-        R: CTypeInfo,
+where
+    R: CTypeInfo,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(vec![], R::type_info());
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
 
-
 impl<T1, R> CTypeInfo for extern "C" fn(T1) -> R
 where
     T1: CTypeInfo,
     R: CTypeInfo,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(vec![Parameter::new("x0".to_string(), T1::type_info())], R::type_info());
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
 
-impl<T1, R> CTypeInfo for Option<extern "C" fn(T1) -> R>
-    where
-        T1: CTypeInfo,
-        R: CTypeInfo,
-{
-    fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.set_rval(R::type_info());
-        CType::FnPointer(FnPointerType::new(sig))
-    }
-}
-
-impl<T1, T2, R> CTypeInfo for extern "C" fn(T1, T2) -> R
+impl<'a, T1, R> CTypeInfo for Option<extern "C" fn(T1) -> R>
 where
-    T1: CTypeInfo,
-    T2: CTypeInfo,
-    R: CTypeInfo,
+    T1: CTypeInfo + 'a,
+    R: CTypeInfo + 'a,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.add_param(Parameter::new("x2".to_string(), T2::type_info()));
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(vec![Parameter::new("x0".to_string(), T1::type_info())], R::type_info());
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
 
-impl<T1, T2, R> CTypeInfo for Option<extern "C" fn(T1, T2) -> R>
-    where
-        T1: CTypeInfo,
-        T2: CTypeInfo,
-        R: CTypeInfo,
-{
-    fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.add_param(Parameter::new("x2".to_string(), T2::type_info()));
-        sig.set_rval(R::type_info());
-        CType::FnPointer(FnPointerType::new(sig))
-    }
-}
-
-
-impl<T1, T2, T3, R> CTypeInfo for extern "C" fn(T1, T2, T3) -> R
+impl<'a, T1, T2, R> CTypeInfo for extern "C" fn(T1, T2) -> R
 where
-    T1: CTypeInfo,
-    T2: CTypeInfo,
-    T3: CTypeInfo,
-    R: CTypeInfo,
+    T1: CTypeInfo + 'a,
+    T2: CTypeInfo + 'a,
+    R: CTypeInfo + 'a,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.add_param(Parameter::new("x2".to_string(), T2::type_info()));
-        sig.add_param(Parameter::new("x3".to_string(), T3::type_info()));
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(
+            vec![Parameter::new("x0".to_string(), T1::type_info()), Parameter::new("x1".to_string(), T1::type_info())],
+            R::type_info(),
+        );
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
 
-impl<T1, T2, T3, R> CTypeInfo for Option<extern "C" fn(T1, T2, T3) -> R>
-    where
-        T1: CTypeInfo,
-        T2: CTypeInfo,
-        T3: CTypeInfo,
-        R: CTypeInfo,
+impl<'a, T1, T2, R> CTypeInfo for Option<extern "C" fn(T1, T2) -> R>
+where
+    T1: CTypeInfo + 'a,
+    T2: CTypeInfo + 'a,
+    R: CTypeInfo + 'a,
 {
     fn type_info() -> CType {
-        let mut sig = FunctionSignature::new();
-        sig.add_param(Parameter::new("x1".to_string(), T1::type_info()));
-        sig.add_param(Parameter::new("x2".to_string(), T2::type_info()));
-        sig.add_param(Parameter::new("x3".to_string(), T3::type_info()));
-        sig.set_rval(R::type_info());
+        let sig = FunctionSignature::new(
+            vec![Parameter::new("x0".to_string(), T1::type_info()), Parameter::new("x1".to_string(), T1::type_info())],
+            R::type_info(),
+        );
+        CType::FnPointer(FnPointerType::new(sig))
+    }
+}
+
+impl<'a, T1, T2, T3, R> CTypeInfo for extern "C" fn(T1, T2, T3) -> R
+where
+    T1: CTypeInfo + 'a,
+    T2: CTypeInfo + 'a,
+    T3: CTypeInfo + 'a,
+    R: CTypeInfo + 'a,
+{
+    fn type_info() -> CType {
+        let sig = FunctionSignature::new(
+            vec![
+                Parameter::new("x0".to_string(), T1::type_info()),
+                Parameter::new("x1".to_string(), T1::type_info()),
+                Parameter::new("x2".to_string(), T2::type_info()),
+            ],
+            R::type_info(),
+        );
+        CType::FnPointer(FnPointerType::new(sig))
+    }
+}
+
+impl<'a, T1, T2, T3, R> CTypeInfo for Option<extern "C" fn(T1, T2, T3) -> R>
+where
+    T1: CTypeInfo + 'a,
+    T2: CTypeInfo + 'a,
+    T3: CTypeInfo + 'a,
+    R: CTypeInfo + 'a,
+{
+    fn type_info() -> CType {
+        let sig = FunctionSignature::new(
+            vec![
+                Parameter::new("x0".to_string(), T1::type_info()),
+                Parameter::new("x1".to_string(), T1::type_info()),
+                Parameter::new("x2".to_string(), T2::type_info()),
+            ],
+            R::type_info(),
+        );
         CType::FnPointer(FnPointerType::new(sig))
     }
 }
