@@ -1,7 +1,8 @@
 //! Like a regular `&[T]` but FFI safe.
 
-use crate::lang::c::{CType, CompositeType, Field, PrimitiveType};
+use crate::lang::c::{CType, CompositeType, Documentation, Field, PrimitiveType, Visibility};
 use crate::lang::rust::CTypeInfo;
+use crate::patterns::TypePattern;
 use std::marker::PhantomData;
 use std::ops::Deref;
 
@@ -64,13 +65,29 @@ impl<'a, T> CTypeInfo for FFISlice<'a, T>
 where
     T: CTypeInfo,
 {
+    #[rustfmt::skip]
     fn type_info() -> CType {
         let fields = vec![
-            Field::new("data".to_string(), CType::ReadPointer(Box::new(T::type_info()))),
-            Field::new("len".to_string(), CType::Primitive(PrimitiveType::U64)),
+            Field::with_documentation("data".to_string(), CType::ReadPointer(Box::new(T::type_info())), Visibility::Private, Documentation::new()),
+            Field::with_documentation("len".to_string(), CType::Primitive(PrimitiveType::U64), Visibility::Private, Documentation::new()),
         ];
 
         let composite = CompositeType::new(format!("FFISlice{}", T::type_info().name_within_lib()), fields);
-        CType::Composite(composite)
+        CType::Pattern(TypePattern::Slice(composite))
+    }
+}
+
+#[cfg(test)]
+mod test {
+    use crate::patterns::slice::FFISlice;
+
+    #[test]
+    fn can_create() {
+        let slice = &[0, 1, 2, 3, 5];
+        let empty = FFISlice::<u8>::empty();
+        let some = FFISlice::<u8>::from_slice(slice);
+
+        assert_eq!(empty.as_slice().unwrap(), &[]);
+        assert_eq!(some.as_slice().unwrap(), slice);
     }
 }
